"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
const _1 = require(".");
function subtract(other, main) {
    var immutable = core_1.isImmutable(main._owner);
    var outputSet = internals_1.cloneSortedMap(true, main);
    if (other && typeof other === 'object') {
        if (Array.isArray(other)) {
            subtractArray(main, outputSet, other);
        }
        else if (core_1.isIterable(other)) {
            if (internals_1.isSortedMap(other)) {
                subtractSortedMap(main, outputSet, internals_1.iterate(other));
            }
            else {
                subtractIterable(main, outputSet, other[Symbol.iterator]());
            }
        }
        if (_1.size(outputSet) === _1.size(main)) {
            return main;
        }
        if (immutable) {
            main = internals_1.refreeze(outputSet);
        }
        else {
            main._keyMap = outputSet._keyMap;
            main._sortedValues = outputSet._sortedValues;
        }
    }
    return main;
}
exports.subtract = subtract;
function subtractArray(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    for (var i = 0; i < omissions.length; i++) {
        var key = Array.isArray(omissions[i]) ? omissions[i][0] : omissions[i];
        internals_1.unsetItem(key, keyMap, sortedValues);
    }
}
function subtractIterable(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    var current;
    while (!(current = omissions.next()).done) {
        var key = Array.isArray(current.value) ? current.value[0] : current.value;
        internals_1.unsetItem(key, keyMap, sortedValues);
    }
}
function subtractSortedMap(inputSet, outputSet, omissions) {
    var { _keyMap: keyMap, _sortedValues: sortedValues } = outputSet;
    var current;
    while (!(current = omissions.next()).done) {
        internals_1.unsetItem(current.value.key, keyMap, sortedValues);
    }
}

//# sourceMappingURL=subtract.js.map
