import { SortedMap } from './internals';
import { KeyedFilterFn as _KeyedFilterFn, KeyedForEachFn as _KeyedForEachFn, KeyedMapFn as _KeyedMapFn, KeyedReduceFn as _KeyedReduceFn } from '@collectable/core';
import { UpdateSetCallback } from './functions';
export { UpdateSetCallback, clone, empty, isSortedMap, freeze, isFrozen, fromArray, fromIterable, size, isEmpty, thaw, isThawed, toArray } from './functions';
export interface SetFn {
    <K, V>(key: K, value: V): (set: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(key: K, value: V, map: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const set: SetFn;
export interface FilterFn {
    <K, V>(fn: _KeyedFilterFn<K, V>): (set: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(fn: _KeyedFilterFn<K, V>, map: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const filter: FilterFn;
export interface ForEachFn {
    <K, V>(f: _KeyedForEachFn<K, V>): (set: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(f: _KeyedForEachFn<K, V>, map: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const forEach: ForEachFn;
export interface HasFn {
    <K, V>(value: V): (set: SortedMap<K, V>) => boolean;
    <K, V>(value: V, set: SortedMap<K, V>): boolean;
}
export declare const has: HasFn;
export interface IntersectFn {
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>): (main: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>, main: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const intersect: IntersectFn;
export interface IsEqualFn {
    <K, V>(set: SortedMap<K, V>): (other: SortedMap<K, V>) => boolean;
    <K, V>(set: SortedMap<K, V>, other: SortedMap<K, V>): boolean;
}
export declare const isEqual: IsEqualFn;
export interface MapFn {
    <K, V, R>(fn: _KeyedMapFn<K, V, R>): (set: SortedMap<K, V>) => SortedMap<K, R>;
    <K, V, R>(fn: _KeyedMapFn<K, V, R>, set: SortedMap<K, V>): SortedMap<K, R>;
}
export declare const map: MapFn;
export interface ReduceFn {
    <K, V, R>(f: _KeyedReduceFn<K, V, R>, seed: R, set: SortedMap<K, V>): R;
    <K, V, R>(f: _KeyedReduceFn<K, V, R>, seed: R): (set: SortedMap<K, V>) => R;
    <K, V, R>(f: _KeyedReduceFn<K, V, R>): (seed: R, set: SortedMap<K, V>) => R;
    <K, V, R>(f: _KeyedReduceFn<K, V, R>): (seed: R) => (set: SortedMap<K, V>) => R;
}
export declare const reduce: ReduceFn;
export interface RemoveFn {
    <K, V>(value: V): (set: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(value: V, map: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const remove: RemoveFn;
export interface SubtractFn {
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>): (main: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>, main: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const subtract: SubtractFn;
export interface UnionFn {
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>): (main: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(other: SortedMap<K, V> | [K, V][] | Iterable<[K, V]>, main: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const union: UnionFn;
export interface UnwrapFn {
    <K, V>(deep: boolean): (set: SortedMap<K, V>) => [K, V][];
    <K, V>(deep: boolean, set: SortedMap<K, V>): [K, V][];
}
export declare const unwrap: UnwrapFn;
export interface UpdateFn {
    <K, V>(callback: UpdateSetCallback<K, V>): (set: SortedMap<K, V>) => SortedMap<K, V>;
    <K, V>(callback: UpdateSetCallback<K, V>, map: SortedMap<K, V>): SortedMap<K, V>;
}
export declare const update: UpdateFn;
